Package: modEvA
Type: Package
Title: Model Evaluation and Analysis
Version: 3.45
Date: 2026-05-21
Authors@R: c(person(given = c("A.", "Marcia"),
                      family = "Barbosa",
                      role = c("aut", "cre"),
                      email = "ana.marcia.barbosa@gmail.com"),
               person(given = c("Jennifer", "A."),
                      family = "Brown",
                      role = "aut"),
               person(given = "Alberto",
                      family = "Jimenez-Valverde",
                      role = "aut"),
               person(given = "Raimundo",
                      family = "Real",
                      role = "aut"),
               person(given = "Oswald",
                      family = c("van", "Ginkel"),
                      role = "ctb"),
               person(given = "Jurica",
                      family = "Levatic",
                      role = "ctb"),
               person(given = "Victoria",
                      family = "Formoso-Freire",
                      role = "ctb"),
               person(given = "Andres",
                      family = "Baselga",
                      role = "ctb"),
               person(given = "Carola",
                      family = "Gomez-Rodriguez",
                      role = "ctb"),
               person(given = c("Carlos", "C."),
                      family = "Ramos",
                      role = "ctb"),
               person(given = c("Jose", "Carlos"),
                      family = "Guerrero",
                      role = "fnd"))
Maintainer: A. Marcia Barbosa <ana.marcia.barbosa@gmail.com>
Imports: effectsize, graphics, grDevices, stats, terra (> 1.5-50)
Description: Analyses species distribution models and evaluates their performance. It includes functions for variation partitioning, extracting variable importance, computing several metrics of model discrimination and calibration performance, optimizing prediction thresholds based on a number of criteria, performing multivariate environmental similarity surface (MESS) analysis, and displaying various analytical plots. Initially described in Barbosa et al. (2013) <doi:10.1111/ddi.12100>.
LazyLoad: yes
LazyData: yes
License: GPL-3
URL: http://modeva.r-forge.r-project.org/
BugReports: https://github.com/AMBarbosa/modEvA/issues
NeedsCompilation: no
Packaged: 2026-05-21 20:16:39 UTC; marcia
Author: A. Marcia Barbosa [aut, cre],
  Jennifer A. Brown [aut],
  Alberto Jimenez-Valverde [aut],
  Raimundo Real [aut],
  Oswald van Ginkel [ctb],
  Jurica Levatic [ctb],
  Victoria Formoso-Freire [ctb],
  Andres Baselga [ctb],
  Carola Gomez-Rodriguez [ctb],
  Carlos C. Ramos [ctb],
  Jose Carlos Guerrero [fnd]
Depends: R (>= 2.10)
Repository: CRAN
Date/Publication: 2026-05-21 21:50:11 UTC
Built: R 4.5.2; ; 2026-05-25 02:13:49 UTC; unix
